"""

This is the main part of ERoaster

"""

# ----------------------------------------------------------------------
# Import functions I need 

from os import popen, environ, system, fork, lstat, chmod, unlink, pipe
from socket import socket, gethostbyname, SOCK_STREAM, AF_INET, gethostname, error
from string import strip, find, split, lower, rjust, atoi, atof
from sys import exit
from types import StringType

from gtk import GtkVBox, GtkNotebook, GtkFrame, GtkLabel, GtkHBox, GtkPixmap, GtkTable
from gtk import GtkEntry, GtkCList, GtkScrolledWindow, GtkButton, GtkRadioButton
from gtk import GtkCheckButton, GtkSpinButton, GtkAdjustment, GtkProgressBar, GtkText
from gtk import GtkHScrollbar, GtkVScrollbar, GtkToolbar, GtkStatusbar, GtkFileSelection

from gtk import create_pixmap_from_xpm, rc_parse, mainloop, mainquit, timeout_add, timeout_remove

from gtk import SHADOW_ETCHED_OUT, FILL, EXPAND, DEST_DEFAULT_ALL, SELECTION_MULTIPLE
from gtk import JUSTIFY_RIGHT, TRUE, FALSE, POLICY_AUTOMATIC, SHRINK, ORIENTATION_HORIZONTAL
from gtk import TOOLBAR_BOTH

from gnome.ui import GnomeApp, GnomeFileEntry, GnomeErrorDialog
from gnome.ui import STOCK_MENU_QUIT, STOCK_MENU_REDO, STOCK_MENU_UNDO, STOCK_MENU_ABOUT
from gnome.ui import UIINFO_ITEM_STOCK, UIINFO_SUBTREE
from gnome.ui import DOCK_ITEM_BEH_NORMAL, DOCK_TOP

from ConfigParser import ConfigParser
from select import select
from time import sleep, time, asctime, localtime

from tools import CheckDirectory, SigChldHandler, beautify, striplist

from constants import title, version, frames, datacd, audiocd, imagecd, options
from constants import iconpath, deffile, True, False, gtkrcfile, ecat
from constants import DataCD, DataCDImage, AudioCD, ImageCD
from constants import socketport, year

import GtkExtra
import GDK
import os
import sys
import signal
import popen2
import re
import copy

# -----------------------------------------------------------------------------
# Main Object

class Application:

    def __init__(self, mkisofssupportsgui):

        # Some necessary Variables
        self.parser = ConfigParser()                        # The Configuration File Parser
        self.writermode = None                              # Data CD or Audio CD
        self.totalaudiosize = 0                             # Total size of audio tracks
        self.totaldatasize = 0                              # Total size of data files
        self.selecteddatasize = 0                           # Size of selected data files
        self.nraudiotracks = 0                              # Number of audio tracks
        self.burnwindow = None                              # There's no burn window at the beginning
        self.mkisofssupportsgui = mkisofssupportsgui        # Does mkisofs support -gui ?

        self.formataudio = re.compile("Track [ \d]+: [ \d]+ of [ \d]+ MB written.$")
        self.formataudiolong = re.compile("Track [ \d]+: [ \d]+ of [ \d]+ MB written \(fifo [ \d]+%\).$")
        self.formatdata = re.compile("Track [ \d]+: [ \d]+ MB written.$")
        self.formatdatalong = re.compile("Track [ \d]+: [ \d]+ MB written. [ \d\.]+% done, estimate finish")
        self.formatdatalongfifo = re.compile("Track [ \d]+: [ \d]+ MB written \(fifo [ \d]+%\). [ \d\.]+% done, estimate finish")
        self.formatdatafifo = re.compile("Track [ \d]+: [ \d]+ MB written \(fifo [ \d]+%\).")
        self.formatignore = re.compile("[\d\.]+% done, estimate|.*\d second")
        self.formatimage = re.compile("[\d\.]+% done, estimate finish")
        
        self.filemenu = [ UIINFO_ITEM_STOCK('Quit', None, self.Destroy, STOCK_MENU_QUIT),]
        self.editmenu = [ UIINFO_ITEM_STOCK('Select all', None, self.SelectAll, STOCK_MENU_REDO),
                          UIINFO_ITEM_STOCK('Unselect all', None, self.UnSelectAll, STOCK_MENU_UNDO) ]
        self.helpmenu = [ UIINFO_ITEM_STOCK('About ...', None, self.About, STOCK_MENU_ABOUT),]
        self.menus = [ UIINFO_SUBTREE('File', self.filemenu),
                       UIINFO_SUBTREE('Edit', self.editmenu),
                       UIINFO_SUBTREE('Help', self.helpmenu)]

	# Initialize the window
	self.window = GnomeApp("ERoaster", "%s %s" % (title, version))
	self.window.connect("destroy", self.Destroy)
	self.window.set_border_width(0)

	# Initialize the Main Box
	self.metabox = GtkVBox()
	self.metabox.show()
        self.window.set_contents(self.metabox)
        
        # Create the menu bar
        self.window.create_menus(self.menus)

	# Create the toolbar
        self.toolbar =  GtkToolbar(ORIENTATION_HORIZONTAL, TOOLBAR_BOTH)
        pix, mask = create_pixmap_from_xpm(self.window, None, "%s/image.xpm" % iconpath)
        self.toolbar.append_item("Create Image", "Create Image", "", GtkPixmap(pix, mask), self.CreateImage)
        pix, mask = create_pixmap_from_xpm(self.window, None, "%s/burn.xpm" % iconpath)
        self.toolbar.append_item("Burn CD", "Burn CD", "", GtkPixmap(pix, mask), self.BurnCD)
        self.toolbar.append_space()
        pix, mask = create_pixmap_from_xpm(self.window, None, "%s/select.xpm" % iconpath)
        self.toolbar.append_item("Select All", "Select All", "", GtkPixmap(pix, mask), self.SelectAll)
        pix, mask = create_pixmap_from_xpm(self.window, None, "%s/unselect.xpm" % iconpath)
        self.toolbar.append_item("Unselect All", "Unselect All", "", GtkPixmap(pix, mask), self.UnSelectAll)
        self.toolbar.append_space()
        pix, mask = create_pixmap_from_xpm(self.window, None, "%s/quit.xpm" % iconpath)
        self.toolbar.append_item("Quit", "Quit ECLiPt Roaster", "", GtkPixmap(pix, mask), self.Destroy)
        self.toolbar.show()
        self.window.add_toolbar(self.toolbar, "My Toolbar", DOCK_ITEM_BEH_NORMAL, DOCK_TOP, 1, 1, 0)

        # Create a status bar
        self.statusbar = GtkStatusbar()
        self.statusbar.show()
        self.window.set_statusbar(self.statusbar)
        self.statusbar.push(self.statusbar.get_context_id("main"), " Welcome to the %s %s (c) %s by Martin Preishuber" % (title, version, year))
        
	# Box which holds the tabs
	self.mainbox = GtkVBox(spacing = 5)
	self.mainbox.set_border_width(5)
	self.metabox.pack_start(self.mainbox)
	self.mainbox.show()

	self.notebook = GtkNotebook()
	self.mainbox.pack_start(self.notebook)
	self.notebook.show()

        self.setstatustip = False
        self.notebook.connect("switch_page", self.ShowStatusbarTips)
        
	# Create the Frames:
	for i in range(len(frames)):
	    self.CreateFrame(frames[i])

	# Define entry fields & stuff which gets set per default
	self.LoadOptions()

	# Show main window
	self.window.show()

        signal.signal(signal.SIGCHLD, SigChldHandler)
        self.setstatustip = True

    # Set tips in the status bar
    def ShowStatusbarTips(self, notebook, object, index):
        if self.setstatustip == True:
            if (index == 0):
                message = "Drag & drop some files to the \"Files / Directories\"-list, click AutoSelect and there ya go ;-)"
            elif (index == 1):
                message = "Drag & drop wave files to the list, order 'em via D&D and voila :)"
            elif (index == 2):
                message = "Select a image, click on \"Image info\" for checking and then burn baby burn !"
            elif (index == 3):
                message = "Set the preferences (which will be saved automatically, of course ;-))"
                
            self.statusbar.push(self.statusbar.get_context_id("main"), " %s" % message)
        
    # Create a box consisting of a Xpm and a Label
    def XpmLabelBox(self, text, xpm = None):
	label = GtkLabel(text)
	label.set_padding(2, 2)
	label.show()

	labelbox = GtkHBox()

        if (xpm != None):
            pix, mask = create_pixmap_from_xpm(self.window, None, xpm)
            pixmap = GtkPixmap(pix, mask)
            pixmap.show()
            labelbox.add(pixmap)

	labelbox.add(label)
	labelbox.show()

	return labelbox

    # Define a About Box
    def About(self, args):
        GnomeAbout(title, version, 'May be distributed under the terms of the GPL2', ['Martin Preishuber'], 'This is a frontend for writing CDs using cdrecord & mkisofs.').show()

    # Create the various frames
    def CreateFrame(self, name):

	frame = GtkFrame(name)
	frame.set_border_width(5)
	frame.set_shadow_type(SHADOW_ETCHED_OUT)
	frame.show()
	
	if (name == datacd):
	    box = self.XpmLabelBox(name, "%s/master.xpm" % iconpath)
	    self.BurnData(frame)
	elif (name == audiocd):
	    box = self.XpmLabelBox(name, "%s/audio.xpm" % iconpath)
	    self.BurnAudio(frame)
	elif (name == imagecd):
	    box = self.XpmLabelBox(name, "%s/iso.xpm" % iconpath)
	    self.BurnImage(frame)
	elif (name == options):
	    box = self.XpmLabelBox(name, "%s/options.xpm" % iconpath)
	    self.Options(frame)
            
        self.notebook.append_page(frame, box)

    # Burn CD (menu entry)
    def BurnCD(self, args):
        if (self.burnwindow != None):
            GnomeErrorDialog("Close the output window first !")
            return
        page = self.notebook.get_current_page()
        if (frames[page] == datacd):
            self.writermode = DataCD
        elif (frames[page] == audiocd):
            self.writermode = AudioCD
        elif (frames[page] == imagecd):
            self.writermode = ImageCD
        elif (frames[page] == options):
            GnomeErrorDialog("You can't burn a CD at this point !")
            return
        self.BurnCDNow()

    # Create Image (menu entry)
    def CreateImage(self, args):
        if (self.imageentry.gtk_entry().get_text() == ""):
            GnomeErrorDialog("Please select a image path in the preferences tab first !")
            return
        if (not os.path.isdir(self.imageentry.gtk_entry().get_text())):
            GnomeErrorDialog("Error: \"%s\" is not a valid directory !" % self.imageentry.gtk_entry().get_text())
            return
        if (self.burnwindow != None):
            GnomeErrorDialog("Close the output window first !")
            return
        page = self.notebook.get_current_page()
        if (frames[page] == datacd):
            self.writermode = DataCDImage
        elif (frames[page] == imagecd) or (frames[page] == audiocd) or (frames[page] == options):
            GnomeErrorDialog("You can't create any image at this point !")
            return
        self.BurnCDNow()

    # Burn Image
    def BurnImageNow(self, args):

        if (self.burnwindow != None):
            GnomeErrorDialog("Close the output window first !")
            return

        self.writermode = ImageCD
        self.BurnCDNow()

    # Burn CD depending of which type
    def BurnCDNow(self):

        speed = self.speed.get_value_as_int()
        
        if (self.channel.get_text() == "") or (self.id.get_text() == "") or (self.lun.get_text() == ""):
            GnomeErrorDialog("Invalid recorder settings !")
            return

        if (self.writermode == DataCD) or (self.writermode == DataCDImage):
            if (len(self.datafiles.selection)) == 0:
                GnomeErrorDialog("You have to select files & directories first.")
                return

            if (self.mkisofssupportsgui == True):
                command = "(mkisofs -gui -v"
            else:
                command = "(mkisofs -v"
            if self.allfiles.active: command = "%s -a" % command
            if self.diralloc.active: command = "%s -D" % command
            if self.fullchar.active: command = "%s -l" % command
            if self.allowdots.active: command = "%s -L" % command
            if self.followlinks.active: command = "%s -f" % command
            if self.joliet.active: command = "%s -J" % command
            if self.ratrock.active: command = "%s -r" % command
            if self.rockridge.active: command = "%s -R" % command
            if self.periods.active: command = "%s -d" % command
            volumeid = self.volumeid.get_text()
            if (volumeid != ""):
                command = "%s -V \"%s\"" % (command, volumeid)
                isofilename = volumeid
            else:
                isofilename = "ECLiPt Roaster - %s" % asctime(localtime(time()))

            if (self.writermode == DataCDImage):
                command = "%s -o \"%s/%s.iso\"" % (command, self.imageentry.gtk_entry().get_text(), isofilename)

            for i in range(len(self.datafiles.selection)):
                filename = self.datafiles.get_text(self.datafiles.selection[i], 0)
                if os.path.isdir(filename):
                    if (filename[-1] == "/"):
                        filename = filename[:-1]
                    filenameshort = split(filename, "/")[-1]
                    command = "%s \"/%s/\"=\"%s\"" % (command, filenameshort, filename)
                else:
                    command = "%s \"%s\"" % (command, filename)

            if (self.writermode == DataCD):
                command = "%s | cdrecord -v" % command
                if self.dummy.active: command = "%s -dummy" % command
                if self.eject.active: command = "%s -eject" % command
                command = "%s fs=%d" % (command, self.fifosize.get_value_as_int() * 1024 * 1024)
                command = "%s dev=%d,%d,%d speed=%d -pad -data -) 2>&1 | %s" % (command, int(self.channel.get_text()), int(self.id.get_text()), int(self.lun.get_text()), speed, ecat)
            elif (self.writermode == DataCDImage):
                command = "%s) 2>&1 | %s" % (command, ecat)

        elif (self.writermode == AudioCD):
            
            if (self.songlist.rows) == 0:
                GnomeErrorDialog("You have to select some audio-files first.")
                return

            command = "cdrecord -audio -v"
            if self.dummy.active: command = "%s -dummy" % command
            if self.eject.active: command = "%s -eject" % command
            command = "%s fs=%d" % (command, self.fifosize.get_value_as_int() * 1024 * 1024)
            command = "%s dev=%d,%d,%d speed=%d" % (command, int(self.channel.get_text()), int(self.id.get_text()), int(self.lun.get_text()), speed)
            if self.swab.active: command = "%s -swab" % command
            if self.pad.active: command = "%s -pad" % command

            line = 0
            try:
                filename = self.songlist.get_text(line, 1)
            except:
                filename = None
            while (filename != None):
                command = "%s \"%s\"" % (command, filename)
                self.nraudiotracks = line + 1                                   # Numbers don't start at 0 !
                line = line + 1
                try:
                    filename = self.songlist.get_text(line, 1)
                except:
                    filename = None
            command = "%s 2>&1 | %s" % (command, ecat)

        elif (self.writermode == ImageCD):
            
            image = self.cdimage.gtk_entry().get_text()
            if (image == "") or (os.path.isdir(image)):
                GnomeErrorDialog("The image file \"%s\" is not valid." % image)
                return

            command = "cdrecord -v"
            if self.dummy.active: command = "%s -dummy" % command
            if self.eject.active: command = "%s -eject" % command
            command = "%s fs=%d" % (command, self.fifosize.get_value_as_int() * 1024 * 1024)
            command = "%s dev=%d,%d,%d speed=%s %s | %s" % (command, int(self.channel.get_text()), int(self.id.get_text()), int(self.lun.get_text()), speed, image, ecat)

     	self.BurnWindow(command)

    # Open a new window which contains the cdrecord & mkisofs output
    def BurnWindow(self, command):

	self.burnwindow = GnomeApp("ERoasterOutput", "%s Output" % title)
	self.burnwindow.connect("destroy", self.BurnWindowDestroy)
	self.burnwindow.set_border_width(0)

	self.burnbox = GtkVBox(spacing = 5)
	self.burnbox.set_border_width(5)
	self.burnbox.show()

        # Create a frame containing tracknumber & progress bar

        frame = GtkFrame(label = " Progress Bar ")
        frame.set_border_width(3)
        frame.set_shadow_type(SHADOW_ETCHED_OUT)
        frame.show()

        table = GtkTable(1, 7)

        label = GtkLabel("Track#:")
        label.show()

        self.track = GtkEntry()
        self.track.set_usize(55, -1)
        self.track.set_editable(FALSE)
        self.track.show()

        self.progressbar = GtkProgressBar()
        self.progressbar.set_usize(180, -1)
        self.progressbar.set_format_string("")
        self.progressbar.set_show_text(TRUE)
        self.progressbar.show()

        fifolabel = GtkLabel("FIFO:")
        fifolabel.show()

        self.fifo = GtkEntry()
        self.fifo.set_usize(40, -1)
        self.fifo.set_editable(FALSE)
        self.fifo.show()

        donelabel = GtkLabel("Done:")
        donelabel.show()
        
        self.estimate = GtkEntry()
        self.estimate.set_usize(160, -1)
        self.estimate.set_editable(FALSE)
        self.estimate.set_text("(not estimated)")
        self.estimate.show()
        
        table.attach(label, 0, 1, 0, 1, 0, 0, 5, 5)
        table.attach(self.track, 1, 2, 0, 1, 0, 0, 5, 5)
        table.attach(self.progressbar, 2, 3, 0, 1, 0, 0, 5, 5)
        table.attach(fifolabel, 3, 4, 0, 1, 0, 0, 5, 5)
        table.attach(self.fifo, 4, 5, 0, 1, 0, 0, 5, 5)
        table.attach(donelabel, 5, 6, 0, 1, 0, 0, 5, 5)
        table.attach(self.estimate, 6, 7, 0, 1, 0, 0, 5, 5)
        
        table.show()
        
        frame.add(table)
        
        self.burnbox.pack_start(frame, expand = FALSE)
        
	# Creata frame containing the rest of the output

	outputframe = GtkFrame(label = " CDRecord & MkIsoFS Output ")
	outputframe.set_border_width(3)
        outputframe.set_shadow_type(SHADOW_ETCHED_OUT)
	outputframe.show()
	
        table = GtkTable(2, 2)
        table.set_border_width(10)
        table.set_row_spacing(0, 2)
        table.set_col_spacing(0, 2)
        table.show()

	outputframe.add(table)
        
        self.cdtext = GtkText()
        self.cdtext.set_usize(585, 250)
        self.cdtext.set_editable(FALSE)
        self.cdtext.set_word_wrap(FALSE)
        table.attach(self.cdtext, 0, 1, 0, 1, 0, 0)
        self.cdtext.show()

        hscrollbar = GtkHScrollbar(self.cdtext.get_hadjustment())
        table.attach(hscrollbar, 0, 1, 1, 2, FILL, 0)
        hscrollbar.show()

        vscrollbar = GtkVScrollbar(self.cdtext.get_vadjustment())
        table.attach(vscrollbar, 1, 2, 0, 1, 0, FILL)
        vscrollbar.show()

	self.burnbox.pack_start(outputframe)

	# Insert a table with one button to close the window

        closeframe = GtkFrame()
        closeframe.set_border_width(3)
        closeframe.set_shadow_type(SHADOW_ETCHED_OUT)
        closeframe.show()

	buttontab = GtkTable(1, 3)

	dummylabel = GtkLabel()
	dummylabel.show()
	buttontab.attach(dummylabel, 0, 1, 0, 1, FILL|EXPAND, 0, 5)

	self.closebutton = GtkButton()
	self.closebox = self.XpmLabelBox(" Please wait ... ", "%s/close.xpm" % iconpath)
	self.closebutton.add(self.closebox)
	self.closebutton.show()
    	buttontab.attach(self.closebutton, 1, 2, 0, 1, 0, 0, 0, 5)

	dummylabel = GtkLabel()
	dummylabel.show()
	buttontab.attach(dummylabel, 2, 3, 0, 1, FILL|EXPAND, 0, 5)

	buttontab.show()

        closeframe.add(buttontab)
        
	self.burnbox.pack_start(closeframe, expand = FALSE)

	self.burnwindow.set_contents(self.burnbox)
        self.burnwindow.show()

        pid = os.fork()
        if (pid == 0):
            connected = FALSE
            while (connected == FALSE):
                try:
                    self.client = socket(AF_INET, SOCK_STREAM)
                    self.client.connect(gethostbyname(gethostname()), socketport)
                    connected = TRUE
                except error, detail:
                    sleep(0.1)
            sys.stderr = sys.stdout
            self.pipe = popen2.popen2(command)
            line = None
            while (line != ""):
                line = self.pipe[0].readline()
                if (line != ""):
                    self.client.send(rjust(str(len(line)), 4))
                    self.client.send(line)
            self.client.close()
            os._exit(0)
        else:
            self.server = socket(AF_INET, SOCK_STREAM)
            self.server.bind(gethostbyname(gethostname()), socketport)
            self.server.listen(1)
            self.connection, self.address = self.server.accept()

        self.delhandler = timeout_add(250, self.TimeOutHandler)

    # Handler to check the pipe output
    def TimeOutHandler(self):
        while 1:
            [iwtd, owtd, ewtd] = select([self.connection], [], [], 0)
            if (len(iwtd) == 0):
                return TRUE
            else:
                packet = self.connection.recv(4)
                if (packet == ""):
                    self.closebutton.remove(self.closebox)
                    self.closebox = self.XpmLabelBox(" Close Window ", "%s/close.xpm" % iconpath)
                    self.closebutton.add(self.closebox)
                    self.closebutton.connect("pressed", self.BurnWindowDestroy)
                    return TRUE
                length = atoi(packet)
                packet = self.connection.recv(length)
                if not packet: return FALSE
                splitted = split(packet, "\n")
                for i in range(len(splitted)):
                    line = strip(splitted[i])
                    if (line != ""):
                        
                        if self.formataudio.match(line):
                            if (self.writermode == AudioCD):
                                self.track.set_text("%s of %s" % (str(atoi(line[6:8])), str(self.nraudiotracks)))
                                formatstr = "%s of %sMB (Total %d MB)" % (strip(line[10:13]), strip(line[17:20]), round(float(self.totalaudiosize) / 1024))
                            else:
                                self.track.set_text("%s" % str(atoi(line[6:8])))
                                formatstr = "%p%% (" + strip(line[10:13]) + " of " + strip(line[17:20]) + " MB)"

                            self.progressbar.set_format_string(formatstr)
                            self.progressbar.update(atof(line[10:13]) / atoi(line[17:20]))
                            
                        elif self.formataudiolong.match(line):
                            if (self.writermode == AudioCD):
                                self.track.set_text("%s of %s" % (str(atoi(line[6:8])), str(self.nraudiotracks)))
                                formatstr = "%s of %sMB (Total %d MB)" % (strip(line[10:13]), strip(line[17:20]), round(float(self.totalaudiosize) / 1024))
                            else:
                                self.track.set_text("%s" % str(atoi(line[6:8])))
                                formatstr = "%p%% (" + strip(line[10:13]) + " of " + strip(line[17:20]) + " MB)"
                            self.progressbar.set_format_string(formatstr)
                            self.progressbar.update(atof(line[10:13]) / atof(line[17:20]))
                            self.fifo.set_text("%s%%" % strip(line[38:41]))
                            
                        elif self.formatdata.match(line):
                            self.track.set_text(str(atoi(line[6:8])))
                            formatstr = "%p%% (" + strip(line[9:13]) + " of " + str(int(round(float(self.selecteddatasize) / 1024))) + " MB)"
                            self.progressbar.set_format_string(formatstr)
                            self.progressbar.update(atof(strip(line[9:13])) / round(float(self.selecteddatasize) / 1024))

                        elif self.formatdatalongfifo.match(line):
                            self.track.set_text(str(atoi(line[6:8])))
                            formatstr = "%p%% (" + strip(line[9:13]) + " of " + str(int(round(float(self.selecteddatasize) / 1024))) + " MB)"
                            self.progressbar.set_format_string(formatstr)
                            self.fifo.set_text("%s%%" % strip(line[31:34]))
                            self.progressbar.update(atof(strip(line[9:13])) / round(float(self.selecteddatasize) / 1024))
                            self.estimate.set_text(line[67:])
                            
                        elif self.formatdatafifo.match(line):
                            self.track.set_text(str(atoi(line[6:8])))
                            formatstr = "%p%% (" + strip(line[9:13]) + " of " + str(int(round(float(self.selecteddatasize) / 1024))) + " MB)"
                            self.progressbar.set_format_string(formatstr)
                            self.progressbar.update(atof(strip(line[9:13])) / round(float(self.selecteddatasize) / 1024))
                            self.fifo.set_text("%s%%" % strip(line[31:34]))

                        elif self.formatdatalong.match(line):
                            self.track.set_text(str(atoi(line[6:8])))
                            formatstr = "%p%% (" + strip(line[9:13]) + " of " + str(int(round(float(self.selecteddatasize) / 1024))) + " MB)"
                            self.progressbar.set_format_string(formatstr)
                            self.progressbar.update(atof(strip(line[9:13])) / round(float(self.selecteddatasize) / 1024))
                            self.estimate.set_text(line[55:])

                        elif self.formatimage.match(line):
                            formatstr = "%p%%"
                            self.progressbar.set_format_string(formatstr)
                            self.progressbar.update(atof(strip(line[:find(line, ".")])) / 100)
                            self.estimate.set_text(line[find(line, ".") + 27:])
                            
                        elif (not self.formatignore.match(line)):
                            self.cdtext.insert_defaults("%s\n" % line)

    # Close the Burn window
    def BurnWindowDestroy(self, args):
        timeout_remove(self.delhandler)
        self.connection.close()
        self.server.close()
	self.burnwindow.destroy()
        self.burnwindow = None

    # Set various options for CDRecord & MkIsoFS
    def Options(self, frame):

        # A table containing various options
        optbox = GtkVBox(spacing = 5)
        optbox.show()

        # Filesystem options
        fsframe = GtkFrame(" Filesystem (Data & Audio) ")
        fsframe.set_border_width(5)
        fsframe.set_shadow_type(SHADOW_ETCHED_OUT)
        fsframe.show()

        # Create a table containing the check buttons
	fstab = GtkTable(2, 6)
	fstab.show()

	self.allfiles = GtkCheckButton("Process all files (obsolete)")
	self.allfiles.show()
	self.diralloc = GtkCheckButton("Disable deep directory relocation")
	self.diralloc.show()
	self.fullchar = GtkCheckButton("Allow full 32 character filenames")
	self.fullchar.show()
	self.allowdots = GtkCheckButton("Allow file to start with '.'")
	self.allowdots.show()
	self.followlinks = GtkCheckButton("Follow symbolic links")
	self.followlinks.show()
	self.joliet = GtkCheckButton("Generate Joliet directory information")
	self.joliet.show()
	self.ratrock = GtkCheckButton("Generate rationalized Rock Ridge directory information")
	self.ratrock.show()
	self.rockridge = GtkCheckButton("Generate Rock Ridge directory information")
	self.rockridge.show()
	self.periods = GtkCheckButton("Omit trailing periods from filenames")
	self.periods.show()
	self.pad = GtkCheckButton("Pad audio tracks to a multiple of 2352 bytes")
	self.pad.show()
	self.swab = GtkCheckButton("Audio data source is byte-swapped (little-endian/Intel)")
	self.swab.show()

        fifolabel = GtkLabel("   FIFO Size: ")
        fifolabel.show()
        self.fifosize = GtkSpinButton(GtkAdjustment(4,2,64,1,10,0), 0, 0)
        self.fifosize.show()
        mblabel = GtkLabel(" MB")
        mblabel.show()
        empty = GtkLabel()
        empty.show()

        fifobox = GtkHBox(spacing = 5)
        fifobox.pack_start(fifolabel, expand = FALSE)
        fifobox.pack_start(self.fifosize, expand = FALSE)
        fifobox.pack_start(mblabel, expand = FALSE)
        fifobox.show()
        
	fstab.attach(self.allfiles, 0, 1, 0, 1, FILL, 0, 5)
	fstab.attach(self.diralloc, 0, 1, 1, 2, FILL, 0, 5)
	fstab.attach(self.fullchar, 0, 1, 2, 3, FILL, 0, 5)
	fstab.attach(self.allowdots, 0, 1, 3, 4, FILL, 0, 5)
	fstab.attach(self.followlinks, 0, 1, 4, 5, FILL, 0, 5)
        fstab.attach(self.periods, 0, 1, 5, 6, FILL, 0, 5)
        
	fstab.attach(self.joliet, 1, 2, 0, 1, FILL, 0, 5)
	fstab.attach(self.ratrock, 1, 2, 1, 2, FILL, 0, 5)
	fstab.attach(self.rockridge, 1, 2, 2, 3, FILL, 0, 5)
	fstab.attach(self.pad, 1, 2, 3, 4, FILL, 0, 5)
	fstab.attach(self.swab, 1, 2, 4, 5, FILL, 0, 5)
        fstab.attach(fifobox, 1, 2, 5, 6, FILL, 0, 0)

        imagelabel = GtkLabel("Image path: ")
        imagelabel.show()

	self.imageentry = GnomeFileEntry("tempimage")
	self.imageentry.set_usize(350, -1)
        self.imageentry.show()

        imagetable = GtkTable(1, 2)
        imagetable.attach(imagelabel, 0, 1, 0, 1, 0, 0, 5, 5)
        imagetable.attach(self.imageentry, 1, 2, 0, 1, 0, 0, 5, 5)
        imagetable.show()
        
        fsbox = GtkVBox(spacing = 5)
        fsbox.pack_start(fstab, expand = FALSE, fill = FALSE)
        fsbox.pack_start(imagetable, expand = FALSE, fill = FALSE)
        fsbox.show()

        fsframe.add(fsbox)
        
        optbox.pack_start(fsframe)

        # Recorder options
        recframe = GtkFrame(" Recorder ")
        recframe.set_border_width(5)
        recframe.set_shadow_type(SHADOW_ETCHED_OUT)
        recframe.show()

        # Create a vertical box containing the SCSI-options, speed and mode checkboxes
	recordbox = GtkVBox(spacing = 10)
	recordbox.show()

        # Create a table containing the SCSI-options
	channel = GtkLabel("Channel: ")
	channel.show()
	id = GtkLabel("        ID: ")
        id.show()
	lun = GtkLabel("      Lun: ")
	lun.show()

	self.channel = GtkEntry()
	self.channel.show()
	self.id = GtkEntry()
	self.id.show()
	self.lun = GtkEntry()
	self.lun.show()

	autobutton = GtkButton()
	box = self.XpmLabelBox(" Auto-Detect Writer ")
	autobutton.add(box)
	autobutton.connect('clicked', self.DetectWriter)
	autobutton.show()

        speedlabel = GtkLabel("Speed: ")
        speedlabel.show()
        self.speed = GtkSpinButton(GtkAdjustment(2,1,64,1,10,0), 0, 0)
        self.speed.show()
        xlabel = GtkLabel("x")
        xlabel.show()
        
        speedbox = GtkHBox(spacing = 5)
        speedbox.pack_start(speedlabel, expand = FALSE)
        speedbox.pack_start(self.speed, expand = FALSE)
        speedbox.pack_start(xlabel, expand = FALSE)
        speedbox.show()
        
	recordtab = GtkTable(4, 3)
	recordtab.attach(channel, 0, 1, 0, 1, FILL, 0, 5)
	recordtab.attach(self.channel, 1, 2, 0, 1, FILL, 0, 5)
	recordtab.attach(id, 0, 1, 1, 2, FILL, 0, 5)
	recordtab.attach(self.id, 1, 2, 1, 2, FILL, 0, 5)
	recordtab.attach(lun, 0, 1, 2, 3, FILL, 0, 5)
	recordtab.attach(self.lun, 1, 2, 2, 3, FILL, 0, 5)
	recordtab.attach(autobutton, 2, 3, 0, 3, SHRINK, 0, 5)
        recordtab.attach(speedbox, 3, 4, 0, 3, 0, 0, 5)
	recordtab.show()
        
        # Create a table containing varions Recorder options
	recopttab = GtkTable(1, 2)
	recopttab.show()

	self.dummy = GtkCheckButton("Dummy operation")
	self.dummy.show()
	self.eject = GtkCheckButton("Eject CD after operation")
	self.eject.show()

	recopttab.attach(self.dummy, 0, 1, 0, 1, FILL, 0, 5)
	recopttab.attach(self.eject, 1, 2, 0, 1, FILL, 0, 5)

	recordbox.pack_start(recordtab, expand = FALSE, fill = FALSE)
	recordbox.pack_start(recopttab, expand = FALSE, fill = FALSE)

        recframe.add(recordbox)

        optbox.pack_start(recframe)
        
        frame.add(optbox)

    # Save options to the config file
    def SaveOptions(self):

        filename = "%s/%s" % (environ["HOME"], deffile)
        file = open(filename, "w")

	# Filesystem options
	file.write("[Filesystem]\n")
        file.write("AllFiles: %s\n" % beautify(self.allfiles.active))
	file.write("DisableDeepRelocation: %s\n" % beautify(self.diralloc.active))
	file.write("FullIso9660Filenames: %s\n" % beautify(self.fullchar.active))
	file.write("AllowLeadingDots: %s\n" % beautify(self.allowdots.active))
	file.write("FollowLinks: %s\n" % beautify(self.followlinks.active))
	file.write("Joliet: %s\n" % beautify(self.joliet.active))
	file.write("RationalRock: %s\n" % beautify(self.ratrock.active))
	file.write("Rock: %s\n" % beautify(self.rockridge.active))
	file.write("OmitPeriod: %s\n" % beautify(self.periods.active))
	file.write("Pad: %s\n" % beautify(self.pad.active))
	file.write("Swab: %s\n" % beautify(self.swab.active))

	# Recorder options
	file.write("\n[Recorder]\n")
        file.write("Speed: %d\n" % self.speed.get_value_as_int())
	file.write("Channel: %s\n" % self.channel.get_text())
        file.write("ID: %s\n" % self.id.get_text())
	file.write("LUN: %s\n" % self.lun.get_text())
	file.write("DummyMode: %s\n" % beautify(self.dummy.active))
	file.write("Eject: %s\n" % beautify(self.eject.active))

	# Miscellaneous options
        file.write("\n[Miscellaneous]\n")
        file.write("FIFO: %d\n" % self.fifosize.get_value_as_int())
        file.write("ImagePath: %s\n" % self.imageentry.gtk_entry().get_text())

        file.close()

    # Load configuration options or set defaults
    def LoadOptions(self):
        filename = "%s/%s" % (environ["HOME"], deffile)

        setDefault = False
        
	if os.path.exists(filename):                        # Load configuration file
            try:
                self.parser.read(filename)

                # Filesystem options
                self.allfiles.set_active(beautify(self.parser.get("Filesystem", "AllFiles")))
                self.diralloc.set_active(beautify(self.parser.get("Filesystem", "DisableDeepRelocation")))
                self.fullchar.set_active(beautify(self.parser.get("Filesystem", "FullIso9660Filenames")))
                self.allowdots.set_active(beautify(self.parser.get("Filesystem", "AllowLeadingDots")))
                self.followlinks.set_active(beautify(self.parser.get("Filesystem", "FollowLinks")))
                self.joliet.set_active(beautify(self.parser.get("Filesystem", "Joliet")))
                self.ratrock.set_active(beautify(self.parser.get("Filesystem", "RationalRock")))
                self.rockridge.set_active(beautify(self.parser.get("Filesystem", "Rock")))
                self.periods.set_active(beautify(self.parser.get("Filesystem","OmitPeriod")))
                self.pad.set_active(beautify(self.parser.get("Filesystem", "Pad")))
                self.swab.set_active(beautify(self.parser.get("Filesystem","Swab")))
	    
                # Recorder options
                self.speed.set_value(atoi(self.parser.get("Recorder", "Speed")))
                self.channel.set_text(self.parser.get("Recorder", "Channel"))
                self.id.set_text(self.parser.get("Recorder", "ID"))
                self.lun.set_text(self.parser.get("Recorder", "LUN"))
                self.dummy.set_active(beautify(self.parser.get("Recorder", "DummyMode")))
                self.eject.set_active(beautify(self.parser.get("Recorder", "Eject")))
                
                # Miscellaneous options
                self.fifosize.set_value(atoi(self.parser.get("Miscellaneous", "FIFO")))
                self.imageentry.gtk_entry().set_text(self.parser.get("Miscellaneous", "ImagePath"))

            except:
                
                setDefault = True

        else:                                               # Set default values

            setDefault = True

        if (setDefault == True):
            sys.stdout.write("Warning: couldn't read settings - setting default values !\n\n")
            
	    # Filesystem options
	    self.allfiles.set_active(1)
	    self.diralloc.set_active(1)
	    self.fullchar.set_active(1)
	    self.allowdots.set_active(1)
	    self.followlinks.set_active(0)
	    self.joliet.set_active(1)
	    self.ratrock.set_active(0)
	    self.rockridge.set_active(1)
	    self.periods.set_active(0)
	    self.pad.set_active(1)
	    self.swab.set_active(1)
	    
	    # Recorder options
            self.speed.set_value(2)
	    self.channel.set_text("00")
	    self.id.set_text("00")
	    self.lun.set_text("00")
	    self.dummy.set_active(0)
	    self.eject.set_active(1)
	    
	    # Miscellaneous options
            self.fifosize.set_value(4)

    # Detect the SCSI writer
    def DetectWriter(self, args):
	pipe = popen2.popen2("cat /proc/scsi/scsi")
	output = striplist(pipe[0].readlines())
        found = False
	for i in range(len(output)):
	    line = output[i]
	    if (line[0:4] == "Host"):
		channelpos = find(line, "Channel: ")
		channel = line[channelpos + 9:channelpos + 11]
		idpos = find(line, "Id: ")
		id = line[idpos + 4:idpos + 6]
		lunpos = find(line, "Lun: ")
		lun = line[lunpos + 5:lunpos + 7]
		
	    if (line[0:4] == "Type") and (find(line, "CD-ROM") != -1):
		self.id.set_text(id)
		self.channel.set_text(channel)
		self.lun.set_text(lun)
                found = True
        if (found == False):
            GnomeErrorDialog("Couldn't auto-detect recorder !\nPlease check /proc/scsi/scsi to get the correct settings for Channel, Id & Lun !")


    # Swap two entries in the Songlist
    def SongListSwap(self, row):
        sourceindex = self.songlist.get_text(self.songlist.focus_row, 0)
        targetindex = self.songlist.get_text(row, 0)
        self.songlist.set_text(self.songlist.focus_row, 0, targetindex)
        self.songlist.set_text(row, 0, sourceindex)
        self.songlist.row_move(self.songlist.focus_row, row)
        
    # When button is pressed move the selected file
    def SongListButtonPressed(self, widget, event=None):
        try:
            row = self.songlist.get_selection_info(event.x, event.y)[0]
        except:
            row = -1

    # Stop moving the file when the mouse button is released
    def SongListButtonReleased(self, widget, event=None):
        try:
            row = self.songlist.get_selection_info(event.x, event.y)[0]
        except:
            row = -1
        if (row != self.songlist.focus_row) and (row != -1) and (self.songlist.focus_row != -1):
            self.SongListSwap(row)
            
    # Move files up & down when "dragging"
    def SongListMotion(self, widget, event=None):
        try:
            row = self.songlist.get_selection_info(event.x, event.y)[0]
        except:
            row = -1
        if (row != self.songlist.focus_row) and (row != -1) and (self.songlist.focus_row != -1):
            self.SongListSwap(row)

    # Drag & Drop - Add file to the Songlist window
    def SongListDragReceive(self, window, context, x, y, data, info, time):
        filenames = split(data.data,"\n")
        for i in range(len(filenames)):
            if (filenames[i][:5] == "file:"):
                self.AddFileToList(filenames[i][5:])
        
    # Burn Audio CDs
    def BurnAudio(self, frame):
        burnbox = GtkVBox(spacing = 5)
        burnbox.show()

        targets = [('text/plain', 0, -1)]

        # Volume ID
        volumeframe = GtkFrame(" Volume ID ")
        volumeframe.set_border_width(5)
        volumeframe.set_shadow_type(SHADOW_ETCHED_OUT)
        volumeframe.show()

        # Create a table containing the Volume ID
        volumetab = GtkTable(1, 2)
        volumetab.show()
        
	label = GtkLabel("Enter Volume ID: ")
	label.show()

	self.volumeidaudio = GtkEntry()
	self.volumeidaudio.show()

        volumetab.attach(label, 0, 1, 0, 1, FILL, 0, 5, 5)
        volumetab.attach(self.volumeidaudio, 1, 2, 0, 1, FILL, 0, 5, 5)

        volumeframe.add(volumetab)

        burnbox.pack_start(volumeframe, expand = FALSE)

        # Create a table containing the wav files
        wavframe = GtkFrame(" Wave files ")
        wavframe.set_border_width(5)
        wavframe.set_shadow_type(SHADOW_ETCHED_OUT)
        wavframe.show()

        wavtable = GtkTable(4, 3)
        wavtable.show()

        self.songlist = GtkCList(3, [ " Nr ", " Filename ", " Size " ])
        self.songlist.show()
        self.songlist.connect("drag_data_received", self.SongListDragReceive)
        self.songlist.drag_dest_set(DEST_DEFAULT_ALL, targets, GDK.ACTION_COPY)
        self.songlist.connect("button_press_event", self.SongListButtonPressed)
        self.songlist.connect("button_release_event", self.SongListButtonReleased)
        self.songlist.connect("motion_notify_event", self.SongListMotion)
        self.songlist.set_selection_mode(SELECTION_MULTIPLE)
        self.songlist.set_column_width(1, 280)
        self.songlist.set_column_justification(2, JUSTIFY_RIGHT)

        listwin = GtkScrolledWindow()
        listwin.set_policy(POLICY_AUTOMATIC, POLICY_AUTOMATIC)
        listwin.add(self.songlist)
        listwin.show()
        
        addbutton = GtkButton()
	box = self.XpmLabelBox(" Add File(s) ")
        box.set_usize(95, -1)
	addbutton.add(box)
	addbutton.connect('clicked', self.AddFile)
	addbutton.show()

        delbutton = GtkButton()
	box = self.XpmLabelBox(" Remove File(s) ")
        box.set_usize(95, -1)
	delbutton.add(box)
	delbutton.connect('clicked', self.DelFile)
	delbutton.show()

        sizelabel = GtkLabel("Total: ")
        sizelabel.show()

        self.audiosize = GtkEntry()
        self.audiosize.set_editable(FALSE)
        self.audiosize.set_usize(60, -1)
        self.audiosize.set_text("0 MB")
        self.audiosize.show()
        
        emptyLabel = GtkLabel()
        emptyLabel.show()
        
        wavtable.attach(listwin, 0, 1, 0, 4, EXPAND|FILL, EXPAND|FILL, 5, 5)
        wavtable.attach(addbutton, 1, 3, 0, 1, 0, 0, 5, 5)
        wavtable.attach(delbutton, 1, 3, 1, 2, 0, 0, 5, 5)
        wavtable.attach(emptyLabel, 1, 3, 2, 3, 0, EXPAND|FILL, 5, 5)
        wavtable.attach(sizelabel, 1, 2, 3, 4, 0, 0, 5, 5)
        wavtable.attach(self.audiosize, 2, 3, 3, 4, FILL, 0, 5, 5)
        wavframe.add(wavtable)
        
        burnbox.pack_start(wavframe)
        
	frame.add(burnbox)

    # Delete selected files from the list of wavefiles
    def DelFile(self, args):
        if (len(self.songlist.selection) == 0):
            GnomeErrorDialog("You have to select files first!")
        else:
            list = copy.copy(self.songlist.selection)
            list.sort()
            list.reverse()
            for i in range(len(list)):
                size = int(self.songlist.get_text(list[i], 2)[:-3])
                self.totalaudiosize = self.totalaudiosize - size
                self.audiosize.set_text("%d MB" % round(float(self.totalaudiosize) / 1024))
                
                self.songlist.remove(list[i])

                # Renumber files
                line = list[i]
                try:
                    index = self.songlist.get_text(line, 0)
                except:
                    index = None
                while (index != None):
                    self.songlist.set_text(line, 0, str(int(self.songlist.get_text(line, 0)) - 1))
                    line = line + 1
                    try:
                        index = self.songlist.get_text(line, 0)
                    except:
                        index = None

    # Actually add the file to the list
    def AddFileToList(self, args):
        if (type(args) == StringType):
            filename = strip(args)
        else:
            filename = strip(self.addfilewin.get_filename())

        found = False
        line = 0
        try:
            listname = self.songlist.get_text(line, 1)
        except:
            listname = None
        while (listname != None):
            if (listname == filename):
                found = True
            line = line + 1
            try:
                listname = self.songlist.get_text(line, 1)
            except:
                listname = None

        if (found == False):
            filesize = round(float(os.path.getsize(filename)) / float(1024))
            self.totalaudiosize = self.totalaudiosize + filesize
            filesize = "%d KB" % filesize
            self.songlist.append([str(line + 1), filename, filesize])
            self.audiosize.set_text("%d MB" % round(float(self.totalaudiosize) / 1024))
        
    # Open the file window to select a list of wavefiles
    def AddFile(self, args):
        self.addfilewin = GtkFileSelection("Select wave files")
        self.addfilewin.ok_button.connect("clicked", self.AddFileToList)
        self.addfilewin.cancel_button.connect("clicked", self.addfilewin.destroy)
        self.addfilewin.show()

    # Drag & Drop - Add file to the datafile window
    def DataDragReceive(self, window, context, x, y, data, info, time):
        filenames = split(data.data,"\n")
        for i in range(len(filenames)):
            if (filenames[i][:5] == "file:"):
                self.AddDataFileToList(filenames[i][5:])

    # Add Files to the Datafile list
    def AddDataFileToList(self, args):
        if (type(args) == StringType):
            filename = strip(args)
        else:
            filename = strip(self.adddatawin.get_filename())

        found = False
        line = 0
        try:
            listname = self.datafiles.get_text(line, 0)
        except:
            listname = None
        while (listname != None):
            if (listname == filename):
                found = True
                listname = None
            line = line + 1
            try:
                listname = self.datafiles.get_text(line, 0)
            except:
                listname = None

        if (found == False):
            if os.path.exists(filename):
                pipe = popen("du -s \"%s\"" % filename)
                output = striplist(pipe.readlines())
                size = atoi(split(output[0])[0])
                self.datafiles.append([filename, "%d KB" % size])
                self.totaldatasize = self.totaldatasize + size
                self.datasize.set_text("%d MB" % round(float(self.totaldatasize) / 1024))
            else:
                GnomeErrorDialog("Warning: \"%s\" doesn't exist!" % filename)
        
    # Open the file window to select a list of wavefiles
    def AddDataFile(self, args):
        self.adddatawin = GtkFileSelection("Select files / directories")
        self.adddatawin.ok_button.connect("clicked", self.AddDataFileToList)
        self.adddatawin.cancel_button.connect("clicked", self.adddatawin.destroy)
        self.adddatawin.show()

    # Delete files from the data file selection
    def DelDataFile(self, args):
        if (len(self.datafiles.selection) == 0):
            GnomeErrorDialog("You have to select files/directories first!")
        else:
            list = copy.copy(self.datafiles.selection)
            list.sort()
            list.reverse()
            for i in range(len(list)):
                size = int(self.datafiles.get_text(list[i], 1)[:-3])
                self.totaldatasize = self.totaldatasize - size
                self.datasize.set_text("%d MB" % round(float(self.totaldatasize) / 1024))
                self.datafiles.remove(list[i])

    # Select a data row
    def DataSelectRow(self, clist, row, column, event):
        size = atoi(self.datafiles.get_text(row, 1)[:-3])
        self.selecteddatasize = self.selecteddatasize + size
        self.datasizesel.set_text("%d MB" % round(float(self.selecteddatasize) / 1024))

    # Unselect a data row
    def DataUnselectRow(self, clist, row, column, event):
        size = atoi(self.datafiles.get_text(row, 1)[:-3])
        self.selecteddatasize = self.selecteddatasize - size
        self.datasizesel.set_text("%d MB" % round(float(self.selecteddatasize) / 1024))

    # Calculate & select optimal directories
    def CalculateOptimize(self, args):
        maxsize = 0
        self.datafiles.unselect_all()
        if (self.cdsize650.active):
            maxsize = 650 * 1024
        elif (self.cdsize700.active):
            maxsize = 700 * 1024
        if (self.totaldatasize <= maxsize):
            self.datafiles.select_all()
        else:

            # Insert size & filename into a valuable data structure
            sizefilename = {}
            sizelist = []
            line = 0
            try:
                name = self.datafiles.get_text(line, 0)
                size = atoi(self.datafiles.get_text(line, 1)[:-3])
            except:
                name = None
            while (name != None):
                if sizefilename.has_key(size):
                    sizefilename[size].append(name)
                else:
                    sizefilename[size] = [name]
                    sizelist.append(size)
                line = line + 1
                try:
                    name = self.datafiles.get_text(line, 0)
                    size = atoi(self.datafiles.get_text(line, 1)[:-3])
                except:
                    name = None
            sizelist.sort()
            sizelist.reverse()

            # Move through all available sizes
            selectedsize = 0
            selectedfilenames = []
            for i in range(len(sizelist)):
                size = sizelist[i]
                filenames = sizefilename[size]
                for j in range(len(filenames)):
                    filename = filenames[j]
                    if ((selectedsize + size) <= maxsize):
                        selectedsize = selectedsize + size
                        selectedfilenames.append(filename)

            # Select files
            line = 0
            try:
                name = self.datafiles.get_text(line, 0)
            except:
                name = None
            while (name != None):
                if (name in selectedfilenames):
                    self.datafiles.select_row(line, 0)
                line = line + 1
                try:
                    name = self.datafiles.get_text(line, 0)
                except:
                    name = None
            
    # Burn Data CDs
    def BurnData(self, frame):
        burnbox = GtkVBox(spacing = 5)
        burnbox.show()

        targets = [('text/plain', 0, -1)]

        # Volume ID
        volumeframe = GtkFrame(" Volume ID ")
        volumeframe.set_border_width(5)
        volumeframe.set_shadow_type(SHADOW_ETCHED_OUT)
        volumeframe.show()

        # Create a table containing the Volume ID
        volumetab = GtkTable(1, 2)
        volumetab.show()
        
	label = GtkLabel("Enter Volume ID: ")
	label.show()

	self.volumeid = GtkEntry()
	self.volumeid.show()

        volumetab.attach(label, 0, 1, 0, 1, FILL, 0, 5, 5)
        volumetab.attach(self.volumeid, 1, 2, 0, 1, FILL, 0, 5, 5)

        volumeframe.add(volumetab)

        burnbox.pack_start(volumeframe, expand = FALSE)

        # Create some fields for selecting the Source Directory & calculating the directory size

        sourceframe = GtkFrame(" Directories ")
        sourceframe.set_border_width(5)
        sourceframe.set_shadow_type(SHADOW_ETCHED_OUT)
        sourceframe.show()

        audiotable = GtkTable(7, 3)
        audiotable.show()
        
        self.datafiles = GtkCList(2, [ " Files / Directories ", " Size "])
        self.datafiles.show()
        self.datafiles.connect("select_row", self.DataSelectRow)
        self.datafiles.connect("unselect_row", self.DataUnselectRow)
        self.datafiles.connect("drag_data_received", self.DataDragReceive)
        self.datafiles.drag_dest_set(DEST_DEFAULT_ALL, targets, GDK.ACTION_COPY)
        self.datafiles.set_selection_mode(SELECTION_MULTIPLE)
        self.datafiles.set_column_width(0, 290)
        self.datafiles.set_column_justification(1, JUSTIFY_RIGHT)
        self.datafiles.set_auto_sort(TRUE)

        listwin = GtkScrolledWindow()
        listwin.set_policy(POLICY_AUTOMATIC, POLICY_AUTOMATIC)
        listwin.add(self.datafiles)
        listwin.show()

        adddirbtn = GtkButton()
        box = self.XpmLabelBox(" Add File(s) ")
        box.set_usize(105, -1)
	adddirbtn.add(box)
	adddirbtn.connect('clicked', self.AddDataFile)
	adddirbtn.show()

        deldirbtn = GtkButton()
	box = self.XpmLabelBox(" Remove File(s) ")
        box.set_usize(105, -1)
	deldirbtn.add(box)
        deldirbtn.connect('clicked', self.DelDataFile)
        deldirbtn.show()

        calcbtn = GtkButton()
	box = self.XpmLabelBox(" AutoSelect ")
        box.set_usize(105, -1)
	calcbtn.add(box)
        calcbtn.connect('clicked', self.CalculateOptimize)
        calcbtn.show()

        emptyLabel = GtkLabel()
        emptyLabel.show()

        sizelabel = GtkLabel("     Total: ")
        sizelabel.show()

        self.datasize = GtkEntry()
        self.datasize.set_editable(FALSE)
        self.datasize.set_usize(60, -1)
        self.datasize.set_text("0 MB")
        self.datasize.show()

        selectsize = GtkLabel("Selected: ")
        selectsize.show()

        self.datasizesel = GtkEntry()
        self.datasizesel.set_editable(FALSE)
        self.datasizesel.set_usize(60, -1)
        self.datasizesel.set_text("0 MB")
        self.datasizesel.show()

        self.cdsize650 = GtkRadioButton(None, "650 MB")
	self.cdsize650.show()
	self.cdsize700 = GtkRadioButton(self.cdsize650, "700 MB")
	self.cdsize700.show()

        audiotable.attach(listwin, 0, 1, 0, 7, EXPAND|FILL, EXPAND|FILL, 5, 5)
        audiotable.attach(adddirbtn, 1, 3, 0, 1, 0, 0, 5, 5)
        audiotable.attach(deldirbtn, 1, 3, 1, 2, 0, 0, 5, 5)
        audiotable.attach(calcbtn, 1, 3, 2, 3, 0, 0, 5, 5)
        audiotable.attach(emptyLabel, 1, 3, 3, 4, 0, EXPAND|FILL, 5)
        audiotable.attach(self.cdsize650, 1, 2, 4, 5, 0, 0)
        audiotable.attach(self.cdsize700, 2, 3, 4, 5, 0, 0)
        audiotable.attach(sizelabel, 1, 2, 5, 6, 0, 0)
        audiotable.attach(self.datasize, 2, 3, 5, 6, 0, 0, 0, 5)
        audiotable.attach(selectsize, 1, 2, 6, 7, 0, 0)
        audiotable.attach(self.datasizesel, 2, 3, 6, 7, 0, 0, 5, 5)
        
        sourceframe.add(audiotable)
        
        burnbox.pack_start(sourceframe)

	frame.add(burnbox)

    # Calculate the size of a directory
    def CalculateSize(self, args):
        directory = self.dirfield.gtk_entry().get_text()
        if (strip(directory) == ""):
            GnomeErrorDialog("Please select a directory first!")
        else:
            pipe = popen("du -s -m \"%s\"" % directory)
            output = striplist(pipe.readlines())
            pipe.close()
            directorysize = split(output[0])[0]
            self.sizefield.set_text("%s MB" % directorysize)
        
    # Close the main window (save options first)
    def Destroy(self, args):
        self.SaveOptions()
	self.window.destroy()
	mainquit()

    # Get the information of the image
    def ImageInfo(self, args):
        image = self.cdimage.gtk_entry().get_text()
        if (image == ""):
            GnomeErrorDialog("Please select a image file first.")
            return
        if (not os.path.exists(image)) or (os.path.isdir(image)):
            GnomeErrorDialog("The image file \"%s\" is not valid." % image)
            return
        pipe = popen2.popen2("isoinfo -d -i \"%s\"" % image)
        output = striplist(pipe[0].readlines())
        for i in range(len(output)):
            line = output[i]
            if (line[:10] == "System id:"): self.sysidtext.set_text(line[11:])
            elif (line[:10] == "Volume id:"): self.volidtext.set_text(line[11:])
            elif (line[:14] == "Volume set id:"): self.volsettext.set_text(line[15:])
            elif (line[:13] == "Publisher id:"): self.pubidtext.set_text(line[14:])
            elif (line[:15] == "Application id:"): self.appidtext.set_text(line[16:])
            elif (line[:22] == "Logical block size is:"): self.blsizetext.set_text(line[23:])
            elif (line[:15] == "Volume size is:"): self.volsizetext.set_text(line[16:])

    # Write a Image
    def BurnImage(self, frame):
        burnbox = GtkVBox(spacing = 5)
        burnbox.show()

        # Select Image
        imageframe = GtkFrame(" Select Image ")
        imageframe.set_border_width(5)
        imageframe.set_shadow_type(SHADOW_ETCHED_OUT)
        imageframe.show()

        # Create a table containing the Image selection
        imagetab = GtkTable(1, 3)
        imagetab.show()
        
	label = GtkLabel("Select CD Image: ")
	label.show()

	self.cdimage = GnomeFileEntry("imageentry")
	self.cdimage.set_usize(300, -1)
	self.cdimage.show()

        infobutton = GtkButton()
	infobox = self.XpmLabelBox(" Image info ")
	infobutton.add(infobox)
	infobutton.connect('clicked', self.ImageInfo)
	infobutton.show()
        
        imagetab.attach(label, 0, 1, 0, 1, FILL, 0, 5, 5)
        imagetab.attach(self.cdimage, 1, 2, 0, 1, FILL, 0, 5, 5)
        imagetab.attach(infobutton, 2, 3, 0, 1, FILL, 0, 5, 5)
        
        imageframe.add(imagetab)
        
        burnbox.pack_start(imageframe, expand = FALSE)

        # Frame which contains the iso information

        infoframe = GtkFrame(" ISO Information ")
        infoframe.set_border_width(5)
        infoframe.set_shadow_type(SHADOW_ETCHED_OUT)
        infoframe.show()

        infotab = GtkTable(5, 5)
        infotab.show()

        sysidlabel = GtkLabel("             System ID:")
        sysidlabel.show()

        self.sysidtext = GtkEntry()
        self.sysidtext.set_usize(130, -1)
        self.sysidtext.set_editable(FALSE)
        self.sysidtext.show()

        infotab.attach(sysidlabel, 0, 1, 0, 1, FILL, 0, 5, 5)
        infotab.attach(self.sysidtext, 1, 2, 0, 1, FILL, 0, 5, 5)

        volidlabel = GtkLabel("   Volume ID:")
        volidlabel.show()

        self.volidtext = GtkEntry()
        self.volidtext.set_usize(130, -1)
        self.volidtext.set_editable(FALSE)
        self.volidtext.show()

        infotab.attach(volidlabel, 2, 3, 0, 1, FILL, 0, 5)
        infotab.attach(self.volidtext, 3, 4, 0, 1, FILL, 0, 5)

        volsetlabel = GtkLabel("      Volume set ID:")
        volsetlabel.show()

        self.volsettext = GtkEntry()
        self.volsettext.set_usize(250, -1)
        self.volsettext.set_editable(FALSE)
        self.volsettext.show()

        infotab.attach(volsetlabel, 0, 1, 1, 2, FILL, 0, 5)
        infotab.attach(self.volsettext, 1, 4, 1, 2, FILL, 0, 5)
        
        pubidlabel = GtkLabel("         Publisher ID:")
        pubidlabel.show()

        self.pubidtext = GtkEntry()
        self.pubidtext.set_usize(250, -1)
        self.pubidtext.set_editable(FALSE)
        self.pubidtext.show()

        infotab.attach(pubidlabel, 0, 1, 2, 3, FILL, 0, 5, 5)
        infotab.attach(self.pubidtext, 1, 4, 2, 3, FILL, 0, 5, 5)
        
        appidlabel = GtkLabel("      Application ID:")
        appidlabel.show()

        self.appidtext = GtkEntry()
        self.appidtext.set_usize(350, -1)
        self.appidtext.set_editable(FALSE)
        self.appidtext.show()

        infotab.attach(appidlabel, 0, 1, 3, 4, FILL, 0, 5)
        infotab.attach(self.appidtext, 1, 4, 3, 4, FILL, 0, 5)

        blsizelabel = GtkLabel("Logical block size:")
        blsizelabel.show()

        self.blsizetext = GtkEntry()
        self.blsizetext.set_usize(80, -1)
        self.blsizetext.set_editable(FALSE)
        self.blsizetext.show()

        emptylabel = GtkLabel()
        emptylabel.show()
        
        infotab.attach(blsizelabel, 0, 1, 4, 5, FILL, 0, 5, 5)
        infotab.attach(self.blsizetext, 1, 2, 4, 5, FILL, 0, 5, 5)
        
        volsizelabel = GtkLabel("Volume size:")
        volsizelabel.show()

        self.volsizetext = GtkEntry()
        self.volsizetext.set_usize(180, -1)
        self.volsizetext.set_editable(FALSE)
        self.volsizetext.show()

        infotab.attach(volsizelabel, 2, 3, 4, 5, FILL, 0, 5)
        infotab.attach(self.volsizetext, 3, 4, 4, 5, FILL, 0, 5)

        infoframe.add(infotab)
        
        burnbox.pack_start(infoframe, expand = FALSE)

	frame.add(burnbox)

    # Select all files either in the audio track or data track window
    def SelectAll(self, args):
        page = self.notebook.get_current_page()
        if (frames[page] == datacd):
            self.datafiles.select_all()
        elif (frames[page] == audiocd):
            self.songlist.select_all()

    # UnSelect all files either in the audio track or data track window
    def UnSelectAll(self, args):
        page = self.notebook.get_current_page()
        if (frames[page] == datacd):
            self.datafiles.unselect_all()
        elif (frames[page] == audiocd):
            self.songlist.unselect_all()
        
    # Main loop
    def main(self):
        rc_parse(gtkrcfile)
	mainloop()

